var AppSignal = (function() {
  // Caching elements.
  var $signalNotification = null;
  var $lnkViewSignals = null;
  var $signalsContainer = null;

  var isAllShown = false;
  var isLicenseExpiring = false;

  var dropdownHandler = '';

  function init() {
    dropdownHandler = DropdownHandler();
    $signalNotification = $('#signalNotification');
    $lnkViewSignals = $('#lnkViewSignals');
    $signalsContainer = $('#signalFeedContainer .dropdown-menu');
    isLicenseExpiring = false;

    addEventsForSignal();
  }

  function displaySignals(signalList, offset) {
    signalList = sortSignalsOnName(signalList);
    if (signalList.length <= PBConstants.MIN_SIGNALS_TO_DISPLAY) {
      $lnkViewSignals.hide();
      populateSignalsDropdown(signalList, offset, signalList.length);
      return;
    } else {
      $lnkViewSignals.show();
      if (isAllShown) {
        $lnkViewSignals.html(i18next.t('constantMsgs.viewLess'));
        populateSignalsDropdown(signalList, offset, signalList.length);
      } else {
        $lnkViewSignals.html(i18next.t('constantMsgs.viewAll'));
        populateSignalsDropdown(signalList, offset, PBConstants.MIN_SIGNALS_TO_DISPLAY);
      }
    }
    animateSignalIcon();
  }

  function addEventsForSignal() {
    $('#lnkViewSignals').on('click', function(e) {
      e.preventDefault();
      e.stopPropagation();
      isAllShown = !isAllShown;
      $(document).trigger('evt.render-signals-dropdown');
    });
  }

  // To add signals in signals dropdown.
  function populateSignalsDropdown(signals, serverOffset, listItemsLength) {
    $signalNotification.empty();

    // Adding no-signals class when there are not signals present in server data.
    if (!signals || signals.length === 0) {
      $signalsContainer.addClass('no-signals');

      // To render no signals text when no signals are present in signals array.
      dropdownHandler.showMessage(i18next.t('constantMsgs.noSignals'), $signalNotification, $lnkViewSignals);
      return;
    }

    // Removing no-signals class if signals length is not zero.
    if ($signalsContainer.hasClass('no-notification')) {
      $signalsContainer.removeClass('no-notification');
    }

    // If signals are found, show the "view all / view less" button.
    $lnkViewSignals.show();

    var signalsHtml = '';

    // Looping over all signals to generate HTML for them.
    for (var i = 0; i <= listItemsLength - 1; i++) {
      var currSignal = signals[i];
      var date = new Date(currSignal.expirationDate * 1000);
      date = date.getFullYear() === '9999' ? new Date(PBConstants.INVALID_DATE) : new Date(date);
      date = handleDate(date, serverOffset);
      signalsHtml += getSignalHtml(currSignal.type, currSignal.name, date);
    }
    $signalNotification.append(signalsHtml);
  }

  // To render date properly.
  function handleDate(dateObj, Offset) {
    var dt = DateTimeHelper.getDateObj(dateObj, Offset, true);
    if (!dt) {
      return '--';
    }
    var date = DateTimeHelper.formatDate(dt);
    date = date.split(' ');
    var now = new Date();
    var utcTimeStamp = new Date(
      now.getUTCFullYear(),
      now.getUTCMonth(),
      now.getUTCDate(),
      now.getUTCHours(),
      now.getUTCMinutes(),
      now.getUTCSeconds(),
      now.getUTCMilliseconds()
    );
    var currentDate = utcTimeStamp.getTime();
    var  expiryDate = dt.getTime();
    if (expiryDate - currentDate <= PBConstants.SUBSCRIPTION_NOTI_TIME) {
      isLicenseExpiring = true;
    }
    if (currentDate)
    return date[0];
  }

  // To generate HTML for signals dropdown.
  function getSignalHtml(type, name, date) {
    var contentString = '';

    // To display lifetime when expiry date of a subscription is 12/21/9999 or else display with Renew link.
    if (!date.includes('9999')) {
      contentString = '<small class="text-muted">' +
      date +
      ' <a href="' +
      PBConstants.RENEW_LINK +
      '" target="_blank">' +
      'Renew</a>' +
      '</small>';
    } else {
      contentString = '<small class="text-muted">Lifetime</small>';
    }
    return (
      '<div class="dropdown-item notify-item">' +
      '<p class="notify-details">' +
      '<span class="title">' +
      type + '-' + name +
      '</span>' +
       contentString +
      '</p>' +
      '</div>'
    );
  }

  function animateSignalIcon() {
    if (isLicenseExpiring) {
      $('.mdi-calendar').addClass('bounce');
    } else {
      $('.mdi-calendar').removeClass('bounce');
    }
  }

  // To sort signals based on name property of object.
  function sortSignalsOnName(list) {
    // If objects are present in array then only sort.
    if (list.length) {
      list.sort(function(a, b) {
        var textA = a.name.toUpperCase();
        var textB = b.name.toUpperCase();
        return (textA < textB) ? -1 : (textA > textB) ? 1 : 0;
      });
    }
    return list;
  }

  return {
    displaySignals: displaySignals,
    init: init
  };
})();
